/* *******************************************************************************
 *   Kenya                                                                       *
 *   Copyright (C) 2004 Tristan Allwood,                                         *
 *                 2004 Matthew Sackman                                          *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The authors can be contacted by email at toa02@doc.ic.ac.uk                 *
 *                                             ms02@doc.ic.ac.uk                 *
 *                                                                               *
 *********************************************************************************/

/*
 * Created on Aug 9, 2004
 *
 */
package mediator.util;

import java.util.ArrayList;
import java.util.List;

import org.wellquite.kenya.stackMachine.misc.AbstractJob;

/**
 * @author Matthew Sackman
 * @version 1
 */
public class StreamHandlerWatcher extends AbstractJob {

    private final List streams = new ArrayList();

    private boolean finished = false;
    
    public synchronized void addStreamHandler(StreamHandler sh) {
        streams.add(sh);
    }

    public void execute() {
        StreamHandler current = null;
        while (!finished) {
            synchronized (this) {
                if (streams.size() == 0) {
                    finished = true;
                    current = null;
                } else {
                    current = (StreamHandler) streams.remove(0);
                }
            }

            if (current != null) {
                synchronized (current) {
                    while (!current.isFinished()) {
                        try {
                            current.wait();
                        } catch (InterruptedException e) {
                            /* silently ignore interruptions */
                        }
                    }
                }
            }
        }

        synchronized (this) {
            finished = true;
            notifyAll();
        }
    }

    public synchronized boolean isFinished() {
        return finished;
    }
}