/* *******************************************************************************
 *   Kenya                                                                       *
 *   Copyright (C) 2004 Tristan Allwood,                                         *
 *                 2004 Matthew Sackman                                          *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The authors can be contacted by email at toa02@doc.ic.ac.uk                 *
 *                                             ms02@doc.ic.ac.uk                 *
 *                                                                               *
 *********************************************************************************/

/*
 * Created on Jul 8, 2004
 *
 */
package org.wellquite.kenya.stackMachine.types.interfaces;


/**
 * Represents a closure which can be stopped mid execution. Implementing classes
 * must look at the current instruction and if that is itself an
 * IHaltableClosure instance then they must call haltExecution() on it when they
 * receive a haltExecution call.
 * 
 * @author Matthew Sackman
 * @version 1
 */
public interface IHaltableClosure extends IAtomicClosure {

    /**
     * Halt the execution of this closure. This closure must itself halt the
     * execution of any closures it is responsible for running if they are
     * haltable.
     *  
     */
    void haltExecution();

    /**
     * The returned closure, when executed halts the execution of this closure
     * and all nested executing closures within it.
     * 
     * @return
     */
    IAtomicClosure returnFromThisClosure();

}